@@ -17,8 +17,10 @@
 #include <gtk/gtk.h>
 
 #include "support.h"
+#include "main.h"
 #include "keyboard.h"
 #include "tutor.h"
+#include "translation.h"
 #include "adaptability.h"
 
 /**********************************************************************
@@ -34,10 +36,12 @@ adapt_draw_random_pattern ()
 {
 	gint i, j, k;
 	gint tidx;
+	gchar *hlp;
 	gchar *utf8_text;
 	gunichar text[WORDS * (MAX_WORD_LEN + 1) + 3];
 	gunichar word[MAX_WORD_LEN + 1];
 
+	hlp = main_preferences_get_string ("interface", "language");
 	for (i = 0; i < LINES; i++)
 	{			/* paragraphs per exercise */
 		tidx = 0;
@@ -49,20 +53,24 @@ adapt_draw_random_pattern ()
 				adapt_create_number (word);
 
 			if (j == 0)
-				word[0] = g_unichar_toupper (word[0]);
+				word[0] = keyb_unichar_toupper (word[0]);
 			else
 				text[tidx++] = L' ';
 
 			for (k = 0; word[k] != L'\0'; k++)
 				text[tidx++] = word[k];
 		}
-		text[tidx++] = L'.';
+		if (g_str_has_prefix (hlp, "ur"))
+			text[tidx++] = URDU_STOP;
+		else if (trans_lang_has_stopmark ())
+			text[tidx++] = L'.';
 		text[tidx++] = L'\n';
 		text[tidx++] = L'\0';
 		utf8_text = g_ucs4_to_utf8 (text, -1, NULL, NULL, NULL);
 		tutor_draw_paragraph (utf8_text);
 		g_free (utf8_text);
 	}
+	g_free (hlp);
 }
 
 /*
@@ -71,6 +79,7 @@ adapt_draw_random_pattern ()
 void
 adapt_create_word (gunichar word[MAX_WORD_LEN + 1])
 {
+	gchar *hlp;
 	gint i, n;
 	gint vlen, clen, slen;
 	gunichar vowels[20];
@@ -85,7 +94,8 @@ adapt_create_word (gunichar word[MAX_WORD_LEN + 1])
 	for (i = 0; i < n; i++)
 	{
 		if ((rand () % 25))
-		{		/* literal */
+		{
+			/* Literal */
 			if (i % 2)	/* vowel */
 				if (rand () % 30)
 					word[i] = vowels[rand () % vlen];
@@ -95,19 +105,24 @@ adapt_create_word (gunichar word[MAX_WORD_LEN + 1])
 				word[i] = consonants[rand () % clen];
 			else
 				word[i] = vowels[rand () % vlen];
-
-			if (i == 0 && !(rand () % 5))	/* capital */
-				word[0] = g_unichar_toupper (word[0]);
+			if (i == 0 && !(rand () % 7))	/* capital */
+				word[0] = keyb_unichar_toupper (word[0]);
 		}
 		else
-		{		/* symbol */
+		{
+			/* Symbol */
 			word[i] = symbols[rand () % slen];
-			if (rand () % 3)
-			{	/* space after symbol */
+			if (rand () % 12)
+			{	/* end of word: space after symbol */
 				word[i + 1] = L'\0';
 				return;
 			}
 		}
+
+		/* Avoid double diacritics */
+		if (i > 0)
+			if (keyb_is_diacritic (word[i - 1]) && keyb_is_diacritic (word[i]))
+				word[i] = vowels[rand () % vlen];
 	}
 	/*
 	 * Last char
@@ -115,7 +130,14 @@ adapt_create_word (gunichar word[MAX_WORD_LEN + 1])
 	if (rand () % 20)
 		word[n] = vowels[rand () % vlen];
 	else
-		word[n] = L',';	/* consonants[rand () % clen]; */
+	{
+		hlp = main_preferences_get_string ("interface", "language");
+		if (g_str_has_prefix (hlp, "ur"))
+			word[n] = URDU_COMMA;
+		else if (trans_lang_has_stopmark ())
+			word[n] = L',';
+		g_free (hlp);
+	}
 
 	/*
 	 * Null terminated unistring
@@ -129,21 +151,24 @@ adapt_create_word (gunichar word[MAX_WORD_LEN + 1])
 void
 adapt_create_number (gunichar ucs4_word[MAX_WORD_LEN + 1])
 {
-	gchar word[10];
-	gunichar *tmp_ucs4;
-	glong n;
+	gint i;
+	gint alen;
+	gboolean arabic;
+	const gchar digits[11] = "0123456789";
+	gunichar altnums[30];
 
-	if (rand () % 3)
-		snprintf (word, MAX_WORD_LEN + 1, "%i", rand () % 3101 - 999);
-	else
-		snprintf (word, MAX_WORD_LEN + 1, "%.2f",
-			  (rand () % 10000 - 2000) + (rand () % 100) / 100.0);
-	word[MAX_WORD_LEN] = '\0';
-
-	tmp_ucs4 = g_utf8_to_ucs4_fast (word, -1, &n);
-	memcpy (ucs4_word, tmp_ucs4, n * sizeof (gunichar));
-	g_free (tmp_ucs4);
-	ucs4_word[n] = L'\0';
+	arabic = TRUE;
+	alen = keyb_get_altnums (altnums);
+	if (alen > 5)
+		arabic = rand () % 7 ? FALSE : TRUE;
+	for (i = 0; i < 4; i++)
+	{
+		if (arabic)
+			ucs4_word[i] = digits[rand () % 10];
+		else
+			ucs4_word[i] = altnums[rand () % alen];
+	}
+	ucs4_word[4] = L'\0';
 }
 
 /**********************************************************************
@@ -162,26 +187,26 @@ adapt_comment (gdouble accuracy)
 	if (accuracy < 50)
 		tmp_str =
 			g_strdup (_
-				  (" You must be kidding.\n" " Your error rate is too high!\n"
-				   " Please do it right.\n"));
+				  (" Your error rate is too high.\n"
+				   " Maybe you are not ready to practice this exercise.\n"
+				   " Did you accomplish most of the basic course lessons?\n"));
 
 	else if (accuracy < 90)
-		tmp_str = g_strdup (_(" You are not ready to practice this exercise.\n"
-				      " Your accuracy rate is below 90 %...\n"
-				      " Try to accomplish most of the basic course lessons.\n"));
+		tmp_str = g_strdup (_(" Your accuracy rate is below 90%...\n"
+				      " Could you please try again to improve it?\n"));
 	else if (accuracy < 95)
 		tmp_str =
 			g_strdup (_
 				  (" You are doing well. But...\n"
-				   " Could you make the accuracy reach at least 95 %?\n"));
-	else if (accuracy < 99)
+				   " Could you make the accuracy reach 95%?\n"));
+	else if (accuracy < 98)
 		tmp_str = g_strdup (_(" You are almost there,"
-				      " but your accuracy rate is still below 99 %.\n"
+				      " but your accuracy rate is still below 98%.\n"
 				      " Try a few more times,"
-				      " so that I can evaluate your velocity.\n"));
+				      " or maybe you're getting upset, so go to other kind of exercise.\n"));
 	else
 		tmp_str = g_strdup (_(" Very good!\n"
-				      " You succeeded with an accuracy rate above 99 %.\n"
+				      " You succeeded with an accuracy rate above 98%.\n"
 				      " Now it is time to increase your velocity.\n"
 				      " Go to the 3rd exercise at the main menu.\n"));
 
