@@ -314,6 +314,8 @@ basic_draw_lesson ()
 				rnd = rand () % len;
 				sentence[idx++] = char_pool[rnd];
 				char_pool[rnd] = char_pool[--len];
+				if (keyb_is_diacritic (sentence[idx-1]))
+					sentence[idx-1] = L' ';
 			}
 			sentence[idx++] = (j < 8) ? L' ' : UPSYM;
 		}
