@@ -80,7 +80,6 @@ on_window_main_destroy (GtkObject * object, gpointer user_data)
 void
 on_button_help_clicked (GtkButton * button, gpointer user_data)
 {
-	gchar text[TEXTMAXLEN];
 	GtkWidget *dialog;
 	GtkLabel *wg_label;
 	GtkTextView *wg_text;
@@ -91,8 +90,9 @@ on_button_help_clicked (GtkButton * button, gpointer user_data)
 	gtk_label_set_text (wg_label, _("Help?"));
 
 	wg_text = GTK_TEXT_VIEW (lookup_widget (dialog, "text_info"));
-	trans_read_text (text, TEXTMAXLEN, "_help.txt");
+	gchar *text = trans_read_text ("_help.txt");
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (wg_text), text, -1);
+	g_free (text);
 
 	gtk_widget_show (dialog);
 	gtk_widget_hide (window_main_);
@@ -105,7 +105,6 @@ on_button_help_clicked (GtkButton * button, gpointer user_data)
 void
 on_button_about_clicked (GtkButton * button, gpointer user_data)
 {
-	gchar text[TEXTMAXLEN];
 	gchar *text1;
 	GtkWidget *dialog;
 	GtkLabel *wg_label;
@@ -117,7 +116,7 @@ on_button_about_clicked (GtkButton * button, gpointer user_data)
 	gtk_label_set_text (wg_label, _("About 'Klavaro'"));
 
 	wg_text = GTK_TEXT_VIEW (lookup_widget (dialog, "text_info"));
-	trans_read_text (text, TEXTMAXLEN, "_about.txt");
+	gchar *text = trans_read_text ("_about.txt");
 	text1 = strstr (text, "(V)");
 	if (text1 == NULL)
 		text1 = g_strconcat ("\n ", _("Version:"), " ", PACKAGE_VERSION, "\n", text, NULL);
@@ -130,6 +129,7 @@ on_button_about_clicked (GtkButton * button, gpointer user_data)
 
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (wg_text), text1, -1);
 	g_free (text1);
+	g_free (text);
 
 	gtk_widget_show (dialog);
 	gtk_widget_hide (window_main_);
@@ -197,15 +197,15 @@ on_button_change_language_clicked (GtkButton * button, gpointer user_data)
 void
 on_button_intro_clicked (GtkButton * button, gpointer user_data)
 {
-	gchar text[TEXTMAXLEN];
 	GtkWidget *dialog_intro;
 	GtkTextView *wg_text;
 
 	dialog_intro = create_dialog_info ();
 
 	wg_text = GTK_TEXT_VIEW (lookup_widget (dialog_intro, "text_info"));
-	trans_read_text (text, TEXTMAXLEN, "_intro.txt");
+	gchar *text = trans_read_text ("_intro.txt");
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (wg_text), text, -1);
+	g_free (text);
 
 	gtk_widget_show (dialog_intro);
 	gtk_widget_hide (window_main_);
@@ -325,10 +325,19 @@ on_radiobutton_4_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 void
 on_text_info_realize (GtkWidget * widget, gpointer user_data)
 {
+	gchar *tmp_font;
 	PangoFontDescription *font_desc;
 
 	/* Change default font throughout the widget */
-	font_desc = pango_font_description_from_string (NORMAL_FONT);
+	tmp_font = main_preferences_get_string ("tutor", "lesson_font");
+	if (tmp_font == NULL)
+	{
+		tmp_font = g_strdup (LESSON_FONT);
+		main_preferences_set_string ("tutor", "lesson_font", tmp_font);
+	}
+	font_desc = pango_font_description_from_string (tmp_font);
+	g_free (tmp_font);
+	//font_desc = pango_font_description_from_string (NORMAL_FONT);
 	gtk_widget_modify_font (widget, font_desc);
 	pango_font_description_free (font_desc);
 }
@@ -711,8 +720,8 @@ on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 				    "background", TUTOR_YELLOW_GREEN, "foreground", TUTOR_BLACK,
 				    NULL);
 
-	gtk_text_buffer_create_tag (buf, "cursor_blink", "background", TUTOR_BLACK, "foreground",
-				    TUTOR_WHITE, NULL);
+	gtk_text_buffer_create_tag (buf, "cursor_blink", "background", TUTOR_YELLOW, "foreground",
+				    TUTOR_BLACK, NULL);
 
 	tmp_font = main_preferences_get_string ("tutor", "lesson_font");
 	if (tmp_font == NULL)
@@ -721,10 +730,11 @@ on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 		main_preferences_set_string ("tutor", "lesson_font", tmp_font);
 	}
 	gtk_text_buffer_create_tag (buf, "lesson_font", "font", tmp_font, NULL);
-	g_free (tmp_font);
 
 	/* Change default font throughout the widget */
-	font_desc = pango_font_description_from_string (NORMAL_FONT);
+	font_desc = pango_font_description_from_string (tmp_font);
+	g_free (tmp_font);
+	//font_desc = pango_font_description_from_string (NORMAL_FONT);
 	gtk_widget_modify_font (widget, font_desc);
 	pango_font_description_free (font_desc);
 
@@ -1116,8 +1126,12 @@ on_entry_mesg_insert_text (GtkEditable * editable,
 			   gchar * new_text, gint new_text_length, gpointer position,
 			   gpointer user_data)
 {
+	gchar *test;
+	gchar *text;
+
 	if (callbacks_shield)
 		return;
+
 	if (mesg_drag_drop)
 	{
 		mesg_drag_drop = FALSE;
@@ -1126,10 +1140,22 @@ on_entry_mesg_insert_text (GtkEditable * editable,
 		gtk_editable_delete_text (editable, 0, -1);
 		callbacks_shield = FALSE;
 
+		if (g_str_has_prefix (new_text, "file://"))
+			test = new_text + 7;
+		else
+			test = new_text;
+
+		if (g_file_test (test, G_FILE_TEST_IS_REGULAR))
+			g_file_get_contents (test, &text, NULL, NULL);
+		else
+			text = g_strdup (new_text);
+
 		if (tutor_get_type () == TT_VELO)
-			velo_text_write_to_file (new_text, TRUE);
+			velo_text_write_to_file (text, TRUE);
 		else if (tutor_get_type () == TT_FLUID)
-			fluid_text_write_to_file (new_text);
+			fluid_text_write_to_file (text);
+
+		g_free (text);
 		return;
 	}
 
@@ -1443,6 +1469,7 @@ on_button_top10_update_clicked (GtkButton * button, gpointer user_data)
 {
 	GtkWidget *wg;
 
+	main_preferences_save ();
 	wg = lookup_widget (window_top10_, "image_top10_update");
 	gtk_image_set_from_stock (GTK_IMAGE (wg), "gtk-yes", GTK_ICON_SIZE_BUTTON);
 	wg = lookup_widget (window_top10_, "notebook_top10");
@@ -1455,6 +1482,7 @@ on_button_top10_publish_clicked (GtkButton * button, gpointer user_data)
 {
 	GtkWidget *wg;
 
+	main_preferences_save ();
 	wg = lookup_widget (window_top10_, "image_top10_publish");
 	gtk_image_set_from_stock (GTK_IMAGE (wg), "gtk-yes", GTK_ICON_SIZE_BUTTON);
 	wg = lookup_widget (window_top10_, "notebook_top10");
@@ -1569,6 +1597,7 @@ on_button_top10_downupdate_clicked (GtkButton * button, gpointer user_data)
 {
 	GtkWidget *wg;
 
+	main_preferences_save ();
 	wg = lookup_widget (window_top10_, "notebook_top10");
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (wg), 1);
 
@@ -1581,6 +1610,7 @@ on_button_top10_host_test_clicked (GtkButton * button, gpointer user_data)
 {
 	GtkWidget *wg;
 
+	main_preferences_save ();
 	wg = lookup_widget (window_top10_, "notebook_top10");
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (wg), 1);
 	g_idle_add ((GSourceFunc) top10_host_test, NULL);
@@ -1714,3 +1744,5 @@ on_window_hints_destroy (GtkObject * object, gpointer user_data)
 {
 	window_hints_ = NULL;
 }
+
+/* Fim */
