@@ -19,8 +19,8 @@ extern GtkWidget *window_tutor_;
 
 struct
 {
+	gint id;
 	gboolean blink;
-	gboolean is_initializing;
 } cursor;
 
 /*******************************************************************************
@@ -38,18 +38,6 @@ cursor_set_blink (gboolean status)
 	cursor.blink = status;
 }
 
-gboolean
-cursor_get_is_initializing ()
-{
-	return (cursor.is_initializing);
-}
-
-void
-cursor_set_is_initializing (gboolean status)
-{
-	cursor.is_initializing = status;
-}
-
 /*******************************************************************************
  * Advance the cursor n positions on the tutor text view
  */
@@ -89,7 +77,9 @@ cursor_advance (gint n)
 
 	if (i == n)
 	{
-		mark = gtk_text_buffer_create_mark (buf, "aux", &new_start, FALSE);
+		end = new_start;
+		gtk_text_iter_forward_line (&end);
+		mark = gtk_text_buffer_create_mark (buf, "aux", &end, FALSE);
 		cursor_out_screen = gtk_text_view_move_mark_onscreen (GTK_TEXT_VIEW (wg), mark);
 		if (cursor_out_screen)
 		{
@@ -98,10 +88,8 @@ cursor_advance (gint n)
 			wg = lookup_widget (window_tutor_, "scrolledwindow_tutor_main");
 			scroll = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (wg));
 			gtk_adjustment_set_value (scroll,
-						  gtk_adjustment_get_value (scroll) + 3 * 23 * (n >
-												0 ?
-												1 :
-												-1));
+						  gtk_adjustment_get_value (scroll) + 8 * 23 *
+						  (n > 0 ? 1 : -1));
 		}
 	}
 	return (i);
@@ -159,9 +147,9 @@ cursor_get_char ()
 gboolean
 cursor_init (gpointer data)
 {
+	cursor.id = 0;
 	cursor.blink = TRUE;
-	cursor.is_initializing = FALSE;
-	cursor_off (NULL);
+	cursor_on (NULL);
 	return FALSE;
 }
 
@@ -171,6 +159,9 @@ cursor_init (gpointer data)
 gboolean
 cursor_on (gpointer data)
 {
+	if (cursor.id)
+		g_source_remove (cursor.id);
+
 	if (cursor.blink == FALSE)
 		return FALSE;
 
@@ -180,7 +171,7 @@ cursor_on (gpointer data)
 		return FALSE;
 	}
 
-	g_timeout_add (TIME_CURSOR_OFF, &cursor_off, NULL);
+	cursor.id = g_timeout_add (TIME_CURSOR_ON, &cursor_off, NULL);
 
 	cursor_switch_on ();
 
@@ -193,6 +184,9 @@ cursor_on (gpointer data)
 gboolean
 cursor_off (gpointer data)
 {
+	if (cursor.id)
+		g_source_remove (cursor.id);
+
 	if (cursor.blink == FALSE)
 		return FALSE;
 
@@ -202,7 +196,7 @@ cursor_off (gpointer data)
 		return FALSE;
 	}
 
-	g_timeout_add (TIME_CURSOR_ON, &cursor_on, NULL);
+	cursor.id = g_timeout_add (TIME_CURSOR_OFF, &cursor_on, NULL);
 
 	cursor_switch_off ();
 
