@@ -10,8 +10,8 @@
 /*
  * Time for cursor blinkings, in miliseconds
  */
-#define TIME_CURSOR_ON 250
-#define TIME_CURSOR_OFF 250
+#define TIME_CURSOR_ON 800
+#define TIME_CURSOR_OFF 400
 
 /*
  * Interface functions
@@ -20,10 +20,6 @@ gboolean cursor_get_blink (void);
 
 void cursor_set_blink (gboolean status);
 
-gboolean cursor_get_is_initializing (void);
-
-void cursor_set_is_initializing (gboolean status);
-
 /*
  * Auxilaiar functions
  */
