@@ -143,7 +143,7 @@ fluid_init_paragraph_list (gchar * list_name)
 	gchar *memory_ok;
 	gchar *tmp_name;
 	gchar *tmp_code;
-	gchar str_1000[1001];
+	gchar str_9000[9001];
 	FILE *fh;
 
 	if (list_name && strcmp (list_name, _("--> Default")) != 0)
@@ -165,7 +165,7 @@ fluid_init_paragraph_list (gchar * list_name)
 	}
 
 	fh = (FILE *) g_fopen (tmp_name, "r");
-	if (fh == NULL && strcmp (par.name, "Default") == 0)
+	if (fh == NULL && g_str_equal (par.name, "Default"))
 		fh = trans_lang_get_similar_file (".paragraphs");
 
 	if (fh)
@@ -174,12 +174,12 @@ fluid_init_paragraph_list (gchar * list_name)
 		par.buffer = g_strdup ("");
 		par.len = 0;
 		g_print ("Paragraphs:\n0");
-		while (fgets (str_1000, 1001, fh))
+		while (fgets (str_9000, 9001, fh))
 		{
-			if ((len = strlen (str_1000)) < 2)
+			len = strlen (str_9000);
+			if (len < 2)
 				continue;
-			len += strlen (par.buffer) + 1;
-			memory_ok = g_try_renew (gchar, par.buffer, len);
+			memory_ok = g_try_renew (gchar, par.buffer, strlen (par.buffer) + len + 2);
 			if (memory_ok)
 				par.buffer = memory_ok;
 			else
@@ -187,7 +187,9 @@ fluid_init_paragraph_list (gchar * list_name)
 				g_print ("\nThere was truncation: memory restrictions...");
 				break;
 			}
-			strcat (par.buffer, str_1000);
+			strcat (par.buffer, str_9000);
+			if (len == 9000)
+				strcat (par.buffer, "\n");
 			par.len++;
 			g_print (" - %i", par.len);
 		}
@@ -279,7 +281,7 @@ fluid_filter_utf8 (gchar * text)
 	 */
 	if (raw.len == 0)
 	{
-		flt.txt = g_strdup_printf ("%i.\n", rand () % 9000);
+		flt.txt = g_strdup_printf ("%i\n", rand () % 9999);
 		return (flt.txt);
 	}
 
@@ -311,8 +313,8 @@ fluid_filter_utf8 (gchar * text)
 			break;
 		/* Test valid utf8 char
 		 */
-		if ((uch = g_utf8_get_char_validated (raw.pt, 16)) == (gunichar) - 1
-		    || uch == (gunichar) - 2)
+		if ((uch = g_utf8_get_char_validated (raw.pt, 16)) == (gunichar) -1
+		    || uch == (gunichar) -2)
 			uch = L' ';
 
 		/* Increase the pointer for the input text
@@ -479,19 +481,19 @@ fluid_comment (gdouble accuracy, gdouble velocity, gdouble fluidness)
 	if (accuracy < 98)
 		tmp_str =
 			g_strdup (_
-				  (" Your accuracy rate is too low.\n"
-				   " I want it greater than 98 %.\n"));
+				  (" Your accuracy rate is not good enough.\n"
+				   " I want it greater than 98%.\n"));
 	else if (velocity < 50)
 		tmp_str =
 			g_strdup (_
-				  (" You type accurately but not fast.\n"
+				  (" You type accurately but not so fast.\n"
 				   " Can you reach 50 WPM?\n"));
 	else if (fluidness < 70)
 		tmp_str = g_strdup (_(" Your rhythm is not so constant. Calm down.\n"
-				      " For now, try to make the fluidness greater than 70 %.\n"));
+				      " For now, try to make the fluidness greater than 70%.\n"));
 	else if (fluidness < 80)
 		tmp_str = g_strdup (_(" You are almost getting there. Type more fluently.\n"
-				      " I want a fluidness greater than 80 %.\n"));
+				      " I want a fluidness greater than 80%.\n"));
 	else if (velocity < 80)
 		tmp_str = g_strdup (_(" Congratulations!\n"
 				      " It seems to me that you are a professional.\n"
