@@ -34,10 +34,75 @@ GtkWidget *window_hints_ = NULL;
 
 const gunichar vowels[] = {
 	L'a', L'e', L'i', L'o', L'u',
-	(gunichar) 1072, (gunichar) 1077, (gunichar) 1080, (gunichar) 1086,
-	(gunichar) 1091,
-	(gunichar) 945, (gunichar) 949, (gunichar) 953, (gunichar) 959,
+	(gunichar) 945,
+	(gunichar) 949,
+	(gunichar) 953,
+	(gunichar) 959,
 	(gunichar) 965,
+	(gunichar) 1072,
+	(gunichar) 1077,
+	(gunichar) 1080,
+	(gunichar) 1086,
+	(gunichar) 1091,
+	L'\0'
+};
+
+// Diacritic chars must never appear one after another
+const gunichar diacritics[] = {
+	// Urdu diacritics
+	(gunichar) 0x0640,
+	(gunichar) 0x064B,
+	(gunichar) 0x064E,
+	(gunichar) 0x064F,
+	(gunichar) 0x0650,
+	(gunichar) 0x0651,
+	(gunichar) 0x0654,
+	(gunichar) 0x0670,
+	// Other arabic diacritics
+	(gunichar) 0x0610,
+	(gunichar) 0x0611,
+	(gunichar) 0x0612,
+	(gunichar) 0x0613,
+	(gunichar) 0x0614,
+	(gunichar) 0x0615,
+	(gunichar) 0x0616,
+	(gunichar) 0x0617,
+	(gunichar) 0x0618,
+	(gunichar) 0x0619,
+	(gunichar) 0x061A,
+	(gunichar) 0x064C,
+	(gunichar) 0x064D,
+	(gunichar) 0x0652,
+	(gunichar) 0x0653,
+	(gunichar) 0x0655,
+	(gunichar) 0x0656,
+	(gunichar) 0x0657,
+	(gunichar) 0x0658,
+	(gunichar) 0x0659,
+	(gunichar) 0x065A,
+	(gunichar) 0x065B,
+	(gunichar) 0x065C,
+	(gunichar) 0x065D,
+	(gunichar) 0x065E,
+	(gunichar) 0x06D6,
+	(gunichar) 0x06D7,
+	(gunichar) 0x06D8,
+	(gunichar) 0x06D9,
+	(gunichar) 0x06DA,
+	(gunichar) 0x06DB,
+	(gunichar) 0x06DC,
+	(gunichar) 0x06DF,
+	(gunichar) 0x06E0,
+	(gunichar) 0x06E1,
+	(gunichar) 0x06E2,
+	(gunichar) 0x06E3,
+	(gunichar) 0x06E4,
+	(gunichar) 0x06E7,
+	(gunichar) 0x06E8,
+	(gunichar) 0x06EA,
+	(gunichar) 0x06EB,
+	(gunichar) 0x06EC,
+	(gunichar) 0x06ED,
 	L'\0'
 };
 
@@ -203,6 +268,20 @@ keyb_is_vowel (gunichar chr)
 	return (FALSE);
 }
 
+/**********************************************************************
+ * Test if chr is a diacritic character 
+ */
+gboolean
+keyb_is_diacritic (gunichar chr)
+{
+	gint i;
+
+	for (i = 0; diacritics[i] != L'\0'; i++)
+		if (chr == diacritics[i])
+			return (TRUE);
+	return (FALSE);
+}
+
 /**********************************************************************
  * Get the set of available vowels of the keyboard
  */
@@ -222,8 +301,11 @@ keyb_get_vowels (gunichar * vows)
 				break;
 		}
 	if (k == 0)
-		for (k = 0; k < 5; k++)
-			vows[k] = keyb.lochars[2][k];
+		for (i = j = 0, k = 5; i < 5 && j < 10; i++, j++)
+		{
+			for (; keyb_is_diacritic (keyb.lochars[2][j]) && j < 12; j++);
+			vows[i] = keyb.lochars[2][j];
+		}
 	return (k);
 }
 
@@ -276,6 +358,47 @@ keyb_get_symbols (gunichar * symbols)
 	return (k);
 }
 
+/**********************************************************************
+ * Get the set of available non-arabic digits in the keyboard
+ */
+gint
+keyb_get_altnums (gunichar * altnums)
+{
+	gint i, j;
+	gint k = 0;
+	gunichar chr;
+
+	for (i = 0; i < 4; i++)
+		for (j = 0; j < KEY_LINE_LEN; j++)
+		{
+			chr = keyb.lochars[i][j];
+			if (g_unichar_isdigit (chr) && chr > 255)
+				altnums[k++] = chr;
+
+			chr = keyb.upchars[i][j];
+			if (g_unichar_isdigit (chr) && chr > 255)
+				altnums[k++] = chr;
+		}
+	return (k);
+}
+
+/**********************************************************************
+ * Get the upper case of a letter, only if it's included in the keyboard (by shift)
+ */
+gunichar
+keyb_unichar_toupper (gunichar uchar)
+{
+	gint i,j;
+	gunichar Uchar;
+
+	Uchar = g_unichar_toupper (uchar);
+	for (i = 0; i < 4; i++)
+		for (j = 0; j < KEY_LINE_LEN; j++)
+			if (uchar == keyb.lochars[i][j] && Uchar == keyb.upchars[i][j])
+				return Uchar;
+	return uchar;
+}
+
 /**********************************************************************
  * Save the custom keyboard layout created by the user
  */
