@@ -13,6 +13,8 @@
 #define KEY_LINE_LEN (14+1)	/* 14 keys + 1 NULL char */
 
 #define UPSYM ((gunichar) 182)
+#define URDU_COMMA ((gunichar) 0x060C)
+#define URDU_STOP ((gunichar) 0x06D4)
 
 /*
  * Interface
@@ -38,12 +40,18 @@ void keyb_get_layout (void);
 
 gboolean keyb_is_vowel (gunichar chr);
 
+gboolean keyb_is_diacritic (gunichar chr);
+
 gint keyb_get_vowels (gunichar * vows);
 
 gint keyb_get_consonants (gunichar * consonants);
 
 gint keyb_get_symbols (gunichar * symbols);
 
+gint keyb_get_altnums (gunichar * altnums);
+
+gunichar keyb_unichar_toupper (gunichar uchar);
+
 void keyb_save_new_layout (void);
 
 void keyb_remove_user_layout (void);
