@@ -15,6 +15,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <curl/curl.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
@@ -44,6 +45,7 @@ struct
 	gchar *user_dir;
 	gchar *data_path;
 	gchar *score_path;
+	gboolean curl_ok;
 } general;
 
 
@@ -75,6 +77,12 @@ main_get_score_path ()
 	return (general.score_path);
 }
 
+gboolean
+main_get_curl_ok ()
+{
+	return (general.curl_ok);
+}
+
 gboolean
 main_preferences_exist (gchar * group, gchar * key)
 {
@@ -123,6 +131,32 @@ main_preferences_set_boolean (gchar * group, gchar * key, gboolean value)
 	g_key_file_set_boolean (preferences, group, key, value);
 }
 
+void
+main_preferences_save ()
+{
+	gchar *tmp_name;
+	FILE *fh;
+
+	/*
+	 * Save preferences
+	 */
+	assert_user_dir ();
+	tmp_name = g_strconcat (general.user_dir, "preferences.ini", NULL);
+	fh = (FILE *) g_fopen (tmp_name, "w");
+	if (fh == NULL)
+		g_warning ("couldn't save your preferences at the user folder:\n %s", tmp_name);
+	else
+	{
+		g_free (tmp_name);
+		tmp_name = NULL;
+		tmp_name = g_key_file_to_data (preferences, NULL, NULL);
+		if (tmp_name != NULL)
+			fputs (tmp_name, fh);
+		fclose (fh);
+	}
+	g_free (tmp_name);
+}
+
 /*******************************************************************************
  * Program
  */
@@ -145,10 +179,13 @@ main (int argc, char *argv[])
 		if (g_str_equal (argv[1], "-a"))
 		{
 			main_initialize_global_variables ();
+			general.curl_ok = curl_global_init (CURL_GLOBAL_WIN32) == CURLE_OK ? TRUE : FALSE;
 			top10_global_rank ();
+			if (general.curl_ok) curl_global_cleanup ();
 			return 0;
 		}
 	}
+	general.curl_ok = curl_global_init (CURL_GLOBAL_WIN32) == CURLE_OK ? TRUE : FALSE;
 
 	gtk_disable_setlocale ();	/* The locale will be set according to user preference... */
 	gtk_init (&argc, &argv);
@@ -330,34 +367,15 @@ main_window_init ()
 void
 main_window_pass_away ()
 {
-	gchar *tmp_name;
-	FILE *fh;
-
-	/*
-	 * Save preferences
-	 */
-	assert_user_dir ();
-	tmp_name = g_strconcat (general.user_dir, "preferences.ini", NULL);
-	fh = (FILE *) g_fopen (tmp_name, "w");
-	if (fh == NULL)
-		g_warning ("couldn't save your preferences at the user folder:\n %s", tmp_name);
-	else
-	{
-		g_free (tmp_name);
-		tmp_name = NULL;
-		tmp_name = g_key_file_to_data (preferences, NULL, NULL);
-		if (tmp_name != NULL)
-			fputs (tmp_name, fh);
-		fclose (fh);
-	}
-	g_free (tmp_name);
+	main_preferences_save ();
 
 	g_rmdir ("tmp/klavaro");
 
 	/*
 	 * Good bye
 	 */
-	g_message ("adiaux!");
+	if (general.curl_ok) curl_global_cleanup ();
+	g_print ("\nAdiaux!\n");
 	gtk_exit (0);
 }
 
@@ -379,9 +397,7 @@ assert_user_dir ()
 
 #ifdef G_OS_UNIX
 		g_mkdir (main_get_user_dir (), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
-#endif
-
-#ifdef G_OS_WIN32
+#else
 		g_mkdir (main_get_user_dir (), 0xFFFF);
 #endif
 
