@@ -30,8 +30,8 @@ GtkWidget *window_stat_ = NULL;
 
 struct
 {
-	gfloat y_data[DATA_POINTS];
-	gfloat x_data[DATA_POINTS];
+	gfloat y_data[DATA_POINTS+1];
+	gfloat x_data[DATA_POINTS+1];
 } plot;
 
 /*******************************************************************************
@@ -93,14 +93,14 @@ plot_draw_chart (gint field)
 	gchar *tmp_name;
 	gchar tmp_str[2000];
 	FILE *fh;
-	gfloat accur[DATA_POINTS];
-	gfloat velo[DATA_POINTS];
-	gfloat fluid[DATA_POINTS];
-	gfloat score[DATA_POINTS];
-	gchar data[DATA_POINTS][20];
-	gchar hour[DATA_POINTS][20];
-	gint nchars[DATA_POINTS];
-	gchar lesson[DATA_POINTS][299];
+	gfloat accur[DATA_POINTS+1];
+	gfloat velo[DATA_POINTS+1];
+	gfloat fluid[DATA_POINTS+1];
+	gfloat score[DATA_POINTS+1];
+	gchar data[DATA_POINTS+1][20];
+	gchar hour[DATA_POINTS+1][20];
+	gint nchars[DATA_POINTS+1];
+	gchar lesson[DATA_POINTS+1][299];
 	GdkColor color;
 	GtkWidget *wg;
 	GtkDataboxValue max = { 0, 0 };
