@@ -111,7 +111,6 @@ tutor_init (TutorType tt_type)
 
 	tutor.type = tt_type;
 	cursor_set_blink (FALSE);
-	cursor_set_is_initializing (FALSE);
 
 
 	/******************************
@@ -250,7 +249,6 @@ tutor_update ()
 	case QUERY_PROCESS_TOUCHS:
 		tutor_message (_("STARTED! => "));
 		gtk_widget_show (lookup_widget (window_tutor_, "button_tutor_restart"));
-		gtk_widget_set_sensitive (lookup_widget (window_tutor_, "button_tutor_font"), TRUE);
 		break;
 
 	case QUERY_END:
@@ -263,8 +261,6 @@ tutor_update ()
 				       ("<<==  End of exercise."
 					" Press [Enter] to start another. ==>> "));
 		gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_restart"));
-		gtk_widget_set_sensitive (lookup_widget (window_tutor_, "button_tutor_font"),
-					  FALSE);
 		break;
 	}
 }
@@ -273,11 +269,12 @@ void
 tutor_update_intro ()
 {
 	gchar *tmp_name;
-	gchar text[4096];
 	GtkWidget *wg;
 	GtkLabel *wg_label;
 	GtkTextView *wg_text;
 	GtkAdjustment *scroll;
+	GtkTextIter start;
+	GtkTextIter end;
 
 	if (tutor.type == TT_BASIC)
 	{
@@ -295,13 +292,16 @@ tutor_update_intro ()
 		tutor_message (_("Press any key to start the exercise. "));
 
 	gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_restart"));
-	gtk_widget_set_sensitive (lookup_widget (window_tutor_, "button_tutor_font"), FALSE);
 
 	tmp_name = g_strconcat ("_", tutor_get_type_name (), "_intro.txt", NULL);
-	trans_read_text (text, 4096, tmp_name);
+	gchar *text = trans_read_text (tmp_name);
+
 	g_free (tmp_name);
 	wg_text = GTK_TEXT_VIEW (lookup_widget (window_tutor_, "text_tutor"));
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (wg_text), text, -1);
+	gtk_text_buffer_get_bounds (gtk_text_view_get_buffer (wg_text), &start, &end);
+	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "lesson_font", &start, &end);
+	g_free (text);
 
 	callbacks_shield = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_tutor_, "entry_mesg")), "");
@@ -392,7 +392,6 @@ tutor_update_start ()
 	gtk_text_buffer_apply_tag_by_name (buf, "lesson_font", &start, &end);
 
 	gtk_widget_hide (lookup_widget (window_tutor_, "button_tutor_restart"));
-	gtk_widget_set_sensitive (lookup_widget (window_tutor_, "button_tutor_font"), TRUE);
 	tutor_message (_("Start typing when you are ready. "));
 	callbacks_shield = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_tutor_, "entry_mesg")), "");
@@ -433,12 +432,8 @@ tutor_process_touch (gunichar user_chr)
 		tutor.ttidx = 0;
 		gtk_text_buffer_get_start_iter (wg_buffer, &start);
 		gtk_text_buffer_place_cursor (wg_buffer, &start);
-		if (cursor_get_is_initializing () == FALSE)
-		{
-			cursor_set_blink (FALSE);
-			cursor_set_is_initializing (TRUE);
-			g_timeout_add (TIME_CURSOR_ON + TIME_CURSOR_OFF + 30, &cursor_init, NULL);
-		}
+		cursor_set_blink (TRUE);
+		cursor_on (NULL);
 		if (tutor.type == TT_BASIC)
 			hints_update_from_char (cursor_get_char ());
 		return;
@@ -483,6 +478,7 @@ tutor_process_touch (gunichar user_chr)
 	if (go_on == FALSE)
 	{
 		cursor_set_blink (FALSE);
+		cursor_off (NULL);
 		tutor.elapsed_time = g_timer_elapsed (tutor.tmr, NULL);
 
 		tutor_calc_stats ();
@@ -490,8 +486,12 @@ tutor_process_touch (gunichar user_chr)
 		tutor_update ();
 		tutor_beep ();
 	}
-	else if (tutor.type == TT_BASIC)
-		hints_update_from_char (cursor_get_char ());
+	else
+	{
+		cursor_on (NULL);
+		if (tutor.type == TT_BASIC)
+			hints_update_from_char (cursor_get_char ());
+	}
 }
 
 /**********************************************************************
@@ -529,7 +529,7 @@ tutor_eval_forward (gunichar user_chr)
 
 
 	/*
-	 * Go forwa and test end of text
+	 * Go forward and test end of text
 	 */
 	if (cursor_advance (1) != 1)
 		return (FALSE);
@@ -683,6 +683,7 @@ tutor_calc_stats ()
 	gchar *tmp_str = NULL;
 	gchar *tmp_str2 = NULL;
 	gchar *tmp_name;
+	gchar *tmp;
 	FILE *fh;
 	time_t tmp_time;
 	struct tm *ltime;
@@ -739,7 +740,7 @@ tutor_calc_stats ()
 	/* Verify if logging is allowed 
 	 */
 	may_log = TRUE;
-	if (tutor.type == TT_VELO || tutor.type == TT_FLUID)
+	if (tutor.type == TT_FLUID)
 		if (tutor.n_touchs - tutor.n_errors < MIN_CHARS_TO_LOG)
 		{
 			gdk_beep (); gdk_beep ();
@@ -832,20 +833,24 @@ tutor_calc_stats ()
 			/* Add results to Top 10
 			 */
 			tmp_name = main_preferences_get_string ("interface", "language");
-			stat.lang[0] = tmp_name[0] == 'C' ? 'e' : tmp_name[0];
-			stat.lang[1] = tmp_name[1] ? tmp_name[i] : 'n';
-			stat.genv = UNIX_OK ? 'x' : 'w';
+			stat.lang[0] = ( (tmp_name[0] == 'C') ? 'e' : tmp_name[0] );
+			stat.lang[1] = ( (tmp_name[0] == 'C') ? 'n' : tmp_name[1] );
+			stat.genv = (UNIX_OK ? 'x' : 'w');
 			stat.when = time (NULL);
 			stat.nchars = tutor.n_touchs - tutor.n_errors;
 			stat.accur = accuracy;
 			stat.velo = velocity;
 			stat.fluid = fluidness;
-
 			stat.score = top10_calc_score (&stat);
-			stat.name_len = strlen (g_get_real_name ());
+
+			g_free (tmp_name);
+			tmp = main_preferences_get_string ("tutor", "keyboard");
+			tmp_name = g_strdup_printf ("%s [%s]", g_get_real_name (), tmp);
+			g_free (tmp);
+			stat.name_len = strlen (tmp_name);
 			if (stat.name_len > MAX_NAME_LEN)
 				stat.name_len = MAX_NAME_LEN;
-			strncpy (stat.name, g_get_real_name (), stat.name_len + 1);
+			strncpy (stat.name, tmp_name, stat.name_len + 1);
 			g_free (tmp_name);
 
 			top10_read_stats (LOCAL, -1);
@@ -927,7 +932,7 @@ tutor_calc_stats ()
 		basic_comment (accuracy);
 		break;
 	case TT_ADAPT:
-		tmp_str2 = g_strdup_printf (tmp_str, minutes, seconds, accuracy, 99);
+		tmp_str2 = g_strdup_printf (tmp_str, minutes, seconds, accuracy, 98);
 		gtk_text_buffer_insert_at_cursor (buf, tmp_str2, strlen (tmp_str2));
 		adapt_comment (accuracy);
 		break;
@@ -980,6 +985,7 @@ tutor_calc_stats ()
 			gtk_text_buffer_insert_at_cursor (buf, contest_ps, strlen (contest_ps));
 			g_free (contest_ps);
 		}
+		break;
 	}
 	g_free (tmp_str);
 	g_free (tmp_str2);
@@ -1023,10 +1029,16 @@ tutor_char_distribution_approved ()
 	g_free (tmp_code);
 	if (!g_file_get_contents (tmp_name, &model.text, NULL, NULL))
 	{
-		g_message ("Can't read file:\n %s\n So, not logging your score.", tmp_name);
 		g_free (tmp_name);
-		return FALSE;
+		tmp_name = trans_lang_get_similar_file_name (".paragraphs");
+		if (!g_file_get_contents (tmp_name, &model.text, NULL, NULL))
+		{
+			g_message ("Can't read file:\n %s\n So, not logging your score.", tmp_name);
+			g_free (tmp_name);
+			return FALSE;
+		}
 	}
+	g_message ("model file: %s", tmp_name);
 	g_free (tmp_name);
 
 	/* Get text under examination
