@@ -20,6 +20,7 @@
 #include "support.h"
 #include "main.h"
 #include "translation.h"
+#include "keyboard.h"
 #include "tutor.h"
 #include "velocity.h"
 
@@ -34,7 +35,7 @@ struct
 {
 	GList *list;
 	gint len;
-	gchar name[21];
+	gchar *name;
 } dict;
 
 /*******************************************************************************
@@ -52,7 +53,8 @@ velo_reset_dict ()
 	g_list_free (dict.list);
 	dict.list = NULL;
 	dict.len = 0;
-	dict.name[0] = '\0';
+	g_free (dict.name);
+	dict.name = NULL;
 }
 
 /**********************************************************************
@@ -61,25 +63,18 @@ velo_reset_dict ()
 void
 velo_init ()
 {
-	gchar *tmp_name;
 	gchar *wl_name;
-	FILE *fh;
 
-	if (dict.len == 0)
+	if (dict.len != 0)
+		return;
+
+	if (main_preferences_exist ("tutor", "word_list"))
 	{
-		if (main_preferences_exist ("tutor", "word_list"))
-		{
-			wl_name = main_preferences_get_string ("tutor", "word_list");
-			tmp_name = g_strconcat (main_get_user_dir (), wl_name, ".words", NULL);
-			if ((fh = (FILE *) g_fopen (tmp_name, "r")))
-			{
-				fclose (fh);
-				velo_init_dict (wl_name);
-			}
-			g_free (wl_name);
-			g_free (tmp_name);
-		}
+		wl_name = main_preferences_get_string ("tutor", "word_list");
+		velo_init_dict (wl_name);
+		g_free (wl_name);
 	}
+
 	if (dict.len == 0)
 		velo_init_dict (NULL);
 }
@@ -90,20 +85,19 @@ velo_init ()
 void
 velo_init_dict (gchar * list_name)
 {
-	static gchar *word_str = NULL;	/* no doubt: this is initialized only at the first call */
+	static gchar *word_buf = NULL;	/* no doubt: this is initialized only at the first call */
+	gchar *word;
+	gchar *tmp_buf;
 	gchar *tmp_name;
 	gchar *tmp_code;
-	gchar str_32[33];
-	guint nwords;
-	FILE *fh;
+	gchar *dict_name;
 
-	if (list_name && strcmp (list_name, _("--> Default")) != 0)
+	if (list_name && !g_str_equal (list_name, _("--> Default")) )
 	{
 		main_preferences_set_string ("tutor", "word_list", list_name);
 		tmp_name = g_strconcat (main_get_user_dir (), list_name, ".words", NULL);
 		g_message ("loading dictionary: %s.words", list_name);
-		strncpy (dict.name, list_name, 20);
-		dict.name[20] = '\0';
+		dict_name = g_strdup (list_name);
 	}
 	else
 	{
@@ -111,93 +105,105 @@ velo_init_dict (gchar * list_name)
 		tmp_code = main_preferences_get_string ("interface", "language");
 		tmp_name = g_strconcat (main_get_data_path (), tmp_code, ".words", NULL);
 		g_message ("loading dictionary: %s.words", tmp_code);
-		strcpy (dict.name, "Default");
 		g_free (tmp_code);
+		dict_name = g_strdup ("Default");
 	}
 
-	fh = (FILE *) g_fopen (tmp_name, "r");
-	if (fh == NULL && strcmp (dict.name, "Default") == 0)
-		fh = trans_lang_get_similar_file (".words");
-
-	if (fh)
+	if (!g_file_test (tmp_name, G_FILE_TEST_IS_REGULAR) && g_str_equal (dict_name, "Default") )
 	{
-		g_free (word_str);
-		g_list_free (dict.list);
-		dict.list = NULL;
-		word_str = g_strdup ("");
-		nwords = 0;
+		g_free (tmp_name);
+		tmp_name = trans_lang_get_similar_file_name (".words");
+		g_message ("not found, loading from file:\n %s", tmp_name);
+	}
+
+	if (g_file_get_contents (tmp_name, &tmp_buf, NULL, NULL))
+	{
+		velo_reset_dict ();
+		dict.name = dict_name;
+		g_free (word_buf);
+		word_buf = tmp_buf;
 		g_print ("Tens of words:\n 0");
-		while (fgets (str_32, 33, fh))
+		while (*tmp_buf != '\0')
 		{
-			str_32[strlen (str_32) - 1] = '\0';
-			word_str = g_strconcat (word_str, " ", str_32, NULL);
-			dict.list = g_list_append (dict.list, strrchr (word_str, ' ') + 1);
-			if (++nwords % 10 == 0)
-				g_print (" - %u", nwords / 10);
+			word = tmp_buf;
+			for (;*tmp_buf != '\n' && *tmp_buf != '\0'; tmp_buf++)
+				;
+			dict.list = g_list_prepend (dict.list, word);
+			dict.len++;
+			if (dict.len % 10 == 0)
+				g_print (" - %u", dict.len / 10);
+			if (*tmp_buf == '\0')
+				break;
+			*tmp_buf++ = '\0';
 		}
-		fclose (fh);
-		dict.len = g_list_length (dict.list);
 		g_print ("\n");
 		g_message ("Dictionary loaded!\n\n");
 	}
 	else
 	{
+		g_free (dict_name);
 		g_message ("could not open the file: %s", tmp_name);
-		g_free (tmp_name);
 		tmp_code = main_preferences_get_string ("interface", "language");
-		if (strcmp (tmp_code, "C") == 0)
-			g_error ("so, we must quit!");
+		if (g_str_equal (tmp_code, "C"))
+			g_error ("something wrong, we must quit!");
 		main_preferences_set_string ("interface", "language", "C");
 		velo_init_dict (list_name);
 		main_preferences_set_string ("interface", "language", tmp_code);
 		g_free (tmp_code);
-		return;
 	}
 	g_free (tmp_name);
 }
 
 /**********************************************************************
- * Draw random words selected from a 'discretionary'
+ * Draw random phrases with words selected from a 'discretionary'
  */
 void
 velo_draw_random_words ()
 {
-	gchar *text;
-	gchar str_32[33];
 	gint i, j;
+	gchar *hlp;
+	gchar *word;
+	struct PARAGRAPH
+	{
+		gchar *text;
+		gsize size;
+		gsize i;
+	} par;
 
-	for (i = 0; i < 4; i++)
-	{			/* 4 paragraphs per exercise */
-		text = NULL;
-		for (j = 0; j < 20; j++)
-		{		/* 20 words per paragraph */
-			velo_get_random_word (str_32);
-			if (!j)
+	par.size = 1024;
+	par.text = g_new (gchar, par.size);
+	for (i = 0; i < 4; i++)	/* 4 paragraphs per exercise */
+	{		
+		par.i = 0;
+		for (j = 0; j < 20; j++) /* 20 words per paragraph */
+		{		
+			word = g_strdup (g_list_nth_data (dict.list, rand () % dict.len));
+			if (j == 0)
+				word[0] = g_ascii_toupper (word[0]);
+
+			if (par.i + strlen (word) + 4 > par.size)
 			{
-				str_32[0] = g_ascii_toupper (str_32[0]);
-				text = g_strconcat (str_32, NULL);
+				par.size += 1024;
+				par.text = g_renew (gchar, par.text, par.size);
 			}
-			else
-				text = g_strconcat (text, " ", str_32, NULL);
+
+			strcpy (par.text + par.i, word);
+			par.i += strlen (word);
+			par.text[par.i++] = ' ';
+			g_free (word);
 		}
-		text = g_strconcat (text, ".\n", NULL);
-		tutor_draw_paragraph (text);
-		g_free (text);
+		par.i--;
+		if (trans_lang_has_stopmark ())
+			par.text[par.i++] = '.';
+		par.text[par.i++] = '\n';
+		par.text[par.i++] = '\0';
+		tutor_draw_paragraph (par.text);
 	}
+	g_free (par.text);
 }
 
 /**********************************************************************
- * Selects one word from a 'discretionary'
- */
-void
-velo_get_random_word (gchar str_32[33])
-{
-	strncpy (str_32, g_list_nth_data (dict.list, rand () % dict.len), 33);
-	str_32[32] = '\0';
-}
-
-/**********************************************************************
- * Takes text and make a list of words, validating as UTF-8
+ * Takes text and make a list of words, one per line, validating as UTF-8
  */
 gchar *
 velo_filter_utf8 (gchar * text)
@@ -230,7 +236,7 @@ velo_filter_utf8 (gchar * text)
 	/* Allocate memory space for the result
 	 */
 	flt.i = 0;
-	flt.len = raw.len + 100;
+	flt.len = raw.len + 1024;
 	flt.txt = g_malloc (flt.len);
 
 	raw.pt = text;
@@ -246,19 +252,19 @@ velo_filter_utf8 (gchar * text)
 
 		/* Read valid utf8 char
 		 */
-		if ((uch = g_utf8_get_char_validated (raw.pt, 16)) == (gunichar) - 1
+		if ((uch = g_utf8_get_char_validated (raw.pt, -1)) == (gunichar) - 1
 		    || uch == (gunichar) - 2)
 			uch = L' ';
 
 		/* Increase the pointer for the input text
 		 */
-		raw.pt = g_utf8_find_next_char (raw.pt, raw.pt + 16);
+		raw.pt = g_utf8_find_next_char (raw.pt, NULL);
 
 		/* Verify memory space of output buffer
 		 */
-		if (flt.i < flt.len - 7)
+		if (flt.i < flt.len - 8)
 		{
-			flt.len += 100;
+			flt.len += 1024;
 			flt.txt = g_realloc (flt.txt, flt.len);
 		}
 
@@ -282,14 +288,13 @@ velo_filter_utf8 (gchar * text)
 	{
 		g_free (flt.txt);
 		flt.txt = g_strdup ("01234\n56789\n43210\n98765\n:-)\n");
-		return (flt.txt);
 	}
 
 	return (flt.txt);
 }
 
 /**********************************************************************
- * Reads the text file "file_name" and write to the dictionary.
+ * Reads the text "text_raw" and write to the dictionary.
  * If overwrite is TRUE, then besides overwriting any .words file,
  * it loads a lesson with the new dictionary.
  */
@@ -362,7 +367,7 @@ velo_create_dict (gchar * file_name, gboolean overwrite)
 		return;
 	}
 
-	dictio_name = g_strdup (strrchr (file_name, DIRSEP) + 1);
+	dictio_name = g_path_get_basename (file_name);
 	dict_path = g_strconcat (main_get_user_dir (), dictio_name, ".words", NULL);
 	assert_user_dir ();
 	if (!(fh_destiny = (FILE *) g_fopen (dict_path, "w")))
@@ -410,12 +415,14 @@ velo_comment (gdouble accuracy, gdouble velocity)
 	 * Comments
 	 */
 	if (accuracy < 97)
-		tmp_str = g_strdup (_(" Your accuracy rate is too low.\n"
-				      " I only can rely on you if it is greater than 97 %.\n Please do it again.\n"));
+		tmp_str = g_strdup (_(" Your accuracy rate is not good enough.\n"
+				      " I only can rely on you if it is greater than 97%.\n"
+				      " Please calm down and try it again.\n"));
 	else if (velocity < 10)
 		tmp_str =
-			g_strdup (_
-				  (" You must be kidding. You are so slow!\n Please do it again and again.\n"));
+			g_strdup (_(" You are just beginning.\n"
+				" Be patient, try it again every day, rest and don't worry so much:\n"
+				" persistence and practice will improve your velocity.\n"));
 	else if (velocity < 20)
 		tmp_str =
 			g_strdup (_
@@ -441,7 +448,8 @@ velo_comment (gdouble accuracy, gdouble velocity)
 				  (" Fast! Are you training for a competition?\n So, try to get 70 WPM!\n"));
 	else if (velocity < 80)
 		tmp_str = g_strdup (_(" Top of \"qwerty\"."
-				      " Now it's time to change to the Dvorak mode.\n Are you afraid of reaching 80 WPM?\n"));
+				      " Now it's time to change to the Dvorak mode.\n"
+				      " Are you afraid of reaching 80 WPM?\n"));
 	else if (velocity < 90)
 		tmp_str = g_strdup (_(" Dvorak mode dominated!\n Can you fly at 90 WPM?\n"));
 	else
